# $Id: Catalog.pm,v 1.16 1996/06/05 22:01:31 raman Exp $
package Pdf::Catalog;
require 5.000;
use Pdf::Obj;
use Pdf::Std qw( message_err pdfDateString );
@ISA = qw( Pdf::Obj );
use strict qw ( subs vars refs );
# {{{ Constructor 

# If passed a reference then simply bless the ref into the current
# package. Otherwise create a new object and initialise it.


sub new {
    my $class = shift;
    my $self = shift;
    if( !defined($self) ) {
	$self = Pdf::Obj->new( 'Type', 'Pages',  'Outlines', 'PageMode',
				 'OpenAction', 'Dests', 'URI' );
	$self->setObjDict( 'Type' => Pdf::CosName->new('Catalog') );
    } elsif( !ref($self) ) {
	message_err( 'Any parameter must be a reference to a PdfObj' );
    }
    bless $self, $class;
}

sub refType {
    return 'PdfCatalog';
}

sub setDict {
    my $self = shift;
    my(%p) = @_;
    my $key;
    my $valtype;

    foreach $key (keys %p) {

	$valtype = eval{ $p{$key}->refType };

	if( $key =~ /^pages/i ) {
	    if( $valtype eq 'PdfPages' ) {
		$self->setObjDict( 'Pages', Pdf::CosRef->new( $p{$key} ) );
	    } else {
		message_err( "Catalog '$key' requires a reference to a Pdf Pages Object" );
	    }

	} elsif( $key =~ /^out/i ) {
	    if( $valtype =~ /^Pdf/ ) {
		$self->setObjDict( 'Outlines', Pdf::CosRef->new( $p{$key} ) );
	    } else {
		message_err( "Catalog '$key' requires a reference to a Pdf Object" );
	    }

	}elsif( $key =~ /^open/i ) {
	    if( $valtype =~ /^Action/ ) {
		$self->setObjDict( 'OpenAction', $p{$key} );
	    } else {
		message_err( "Catalog '$key' requires a reference to an Action/Dest" );
	    }

	} elsif( $key =~ /^th/i ) {
	    if( $valtype =~ /^Pdf/ ) {
		$self->setObjDict( 'Threads', Pdf::CosRef->new( $p{$key} ) );
	    } else {
		message_err( "Catalog '$key' requires a reference to a Pdf Object" );
	    }

	} elsif( $key =~ /^de/i ) {
	    if( $valtype eq 'PdfDests' ) {
		$self->setObjDict( 'Dests', Pdf::CosRef->new( $p{$key} ) );
	    } else {
		message_err( "Catalog '$key' requires a reference to a Pdf Dests Object" );
	    }

	} elsif( $key =~ /^pagemode/i ) {
            message_err( "Unrecognised $key => '$p{$key}'" )
              unless ($p{$key} eq 'UseNone'
                      || $p{$key} eq 'UseOutlines'
                      || $p{$key} eq 'UseThumbs'
                      || $p{$key} eq 'FullScreen');
	    $self->setObjDict( 'PageMode', Pdf::CosName->new( $p{$key} ) );

	} elsif( $key =~ /^ur/i ) {
	    if( $valtype eq 'CosDict' ) {
		$self->setObjDict( 'URI', $p{$key} );
	    } elsif( $valtype =~ /^Pdf/ ) {
		$self->setObjDict( 'URI', Pdf::CosRef->new( $p{$key} ) );
	    } else {
		message_err( 'Catalog /URI requires a reference to a Cos Dict or Pdf Object' );
	    }

	} else {
	    message_err( "Key '$key' does not exist in Catalog" );
	}
    }
}



sub validate {
    my $self = shift;

    my $key;
    foreach $key ( 'Pages' ) {
	defined( $self->{$key} ) ||
	    message_err( "Required Catalog entry /$key is missing" );
    }
}

# }}}
# {{{ Emacs local variables  

### Local variables:
### major-mode: perl-mode
### perl-indent-level: 4
### voice-lock-mode: t
### folded-file: t
### End:

# }}}

1;
