package Pdf::ColorMap;
require 5.000;
use Pdf::ColorSpace;
use Pdf::Std qw( message_err f4 assertFloatRange );
use strict qw ( subs vars refs );

# $Id: ColorMap.pm,v 1.9 1996/05/31 15:50:22 raman Exp $
# {{{ Define new color sets

sub new {
    my $class = shift;
    my $self = {};
    bless $self, $class;
}

sub refType {
    'ColorMap';
}

# Add a color map entry to a PdfColorMap object

sub addColorMap {
    my $self = shift;
    my $file = shift;
    my($name,$cspace,@values) = @_;

    # Verify that $cspace is defined and that array depth is correct

    # print "cspace = $cspace, name = $name, values = @values\n";

    my $dimen = $file->{ColorSpace}{$cspace}->{Dimen};
    defined( $dimen ) || message_err( "Colorspace '$cspace' not yet defined." );
    ( (@values) == $dimen ) || 
	message_err( "Incorrect number of array values for colorspace '$cspace'" );
	 
    # Verify that all @values are decimal numbers

    ( assertFloatRange( 0, 1, @values ) == scalar(@values) ) || 
	message_err( "Color array '@values' contains illegal values" );

    # Now register color for later use
    
    $self->{$name}{ColorSpace} = $cspace;
    $self->{$name}{Value} = [ @values ];
}

# Returns a string

sub getColorSpace {
    my $self = shift;
    my $name = shift;
    $self->{$name}{ColorSpace};
}

# Returns a reference to an array

sub getColorValue {
    my $self = shift;
    my $name = shift;
    $self->{$name}{Value};
}

# }}}
# {{{ package Pdf::File

package Pdf::File;

# Used only on startup to give a number of predefined color packages.

sub initColorPackage {
    my $self = shift;
    my($which) = shift;

    my $cmap = Pdf::ColorMap->new();

    if( $which =~ /^defaultrgb$/i ) {
	$cmap->addColorMap( $self, 'black', 'DeviceGray', 0 );
	$cmap->addColorMap( $self, 'white', 'DeviceGray', 1 );
	$cmap->addColorMap( $self, 'red', 'DeviceRGB', 1, 0, 0 );
	$cmap->addColorMap( $self, 'green', 'DeviceRGB', 0, 1, 0 );
	$cmap->addColorMap( $self, 'blue', 'DeviceRGB', 0, 0, 1 );
	$cmap->addColorMap( $self, 'darkblue', 'DeviceRGB', 0, 0, 0.5 );
	$cmap->addColorMap( $self, 'yellow', 'DeviceRGB', 1, 1, 0 );
	$cmap->addColorMap( $self, 'gray', 'DeviceGray', 0.5 );

    } elsif( $which =~ /^defaultgray$/i ) {
	$cmap->addColorMap( $self, 'black', 'DeviceGray', 0 );
	$cmap->addColorMap( $self, 'white', 'DeviceGray', 1 );
	
    } else {
	; # do nothing
    }

    $self->setColorMapRef( $cmap );
    $self->setColorMapRef2( $cmap );
    $cmap;
}

# ColorMap is the colormap used for all drawing and rendering within a
# stream. ColorMap2 is used for Annotations and such that must work in
# DeviceRGB space.

# Set ColorMap and ColorMap2 references in PdfFile object.

sub setColorMapRef {
    my $self = shift;
    my $mapref = shift;
    eval{ $mapref->refType } eq 'ColorMap' ||
	message_err( 'Ref must point to a defined PdfColorMap' );
    $self->{ColorMap} = $mapref;
}

sub setColorMapRef2 {
    my $self = shift;
    my $mapref = shift;
    eval{ $mapref->refType } eq 'ColorMap' ||
	message_err( 'Ref must point to a defined PdfColorMap' );
    $self->{ColorMap2} = $mapref;
}

# Access the ColorSpace and Value for a given color name. For example
# 'red' might return ( 'DeviceRGB', 1, 0, 0 ).

sub getColorMap {
    my $self = shift;
    my $name = shift;
    ( $self->{ColorMap}->{$name}{ColorSpace}, @{$self->{ColorMap}->{$name}{Value}} );
}

sub getColorMap2 {
    my $self = shift;
    my $name = shift;
    ( $self->{ColorMap2}->{$name}{ColorSpace}, @{$self->{ColorMap2}->{$name}{Value}} );
}

# }}}
# {{{ Emacs local variables  

### Local variables:
### major-mode: perl-mode
### perl-indent-level: 4
### voice-lock-mode: t
### folded-file: t
### End:

# }}}

1;
