# $Id: File.pm,v 1.25 1996/05/31 15:58:39 raman Exp $
package Pdf::File;
require 5.000;
use Pdf::Obj;
use Pdf::Info;
use Pdf::Catalog;
use Pdf::Std qw( message_err );

require Exporter;
@ISA = qw(Exporter);
@EXPORT_OK = qw(new #constructor
                pdfOpen pdfClose #opening and closing pdf output 
                beginObj endObj #PDF indirect objects
                reserveObj lateObj #for delayed objects
                setPageSize  #setting page size 
                pdfPrint #Writing  to the PDF output 
                );
use strict qw ( subs vars refs );
# {{{  new init initGlobals

sub new {
    my $class = shift;
    my $glob = shift || \*PDFOUT;
    my $self = {"FileHandle" => $glob};
    bless $self, $class;
    $self->initGlobals;
    $self->init;
    $self;
}

sub refType {
    'File';
}

# Globals static variables

sub initGlobals {
    my $self = shift;
    # Globals statics are prepended with 'pdf_'.

    # If this parameter is 1 then any changes to gstate (eg.
    # setLinewidth) will be written to the stream prior to closing the
    # graphics stream.

    $self->{pdf_stream_flush_gstate} = 0;

    # Defines the initial color package. See initColorPackage() in
    # ColorMap.pm

    $self->{pdf_color_package} = 'defaultrgb';

    # If no page size is passed to pdfClose() then these values are
    # used.

    $self->{pdf_xpsize} = 612;
    $self->{pdf_ypsize} = 792;

    $self->{pdf_make_file_backup} = 1; # if non zero then backup pdf file if
				# creating a new pdf file

    # Private (do not change these values)

    $self->{pdf_page_limit} = 99; # max number of pages per pdf file
    $self->{pdf_language_version} = '1.1'; # pdf language version for file header

    # It is a condition of use of this package that the following lines not be modified.
    
    $self->{pdf_package_version} = '1.0a24';

    $self->{pdf_info_producer} = 'PDF on the Fly v' . $self->{pdf_package_version};
}

# Global variables, init for each new file

sub init {
    my $self = shift;
    
    # Reset incrementing counter of bytes in pdf file.

    $self->{ByteCount} = 0;

    # Reset incrementing counter of assigned objects in pdf file
    
    $self->{ObjCount} = 0;

    # Reset counters of assigned fonts and colorspaces. These indices 
    # are used to make new font and colorspace names (eg. F1, F2, CS1, CS2).

    $self->{FontIndex} = 0;
    $self->{PageIndex} = 0;

    # Create a catalog object. This is done now so that any objects
    # that are created will have somewhere to hang from, regardless of
    # whether the PDF file has been opened or not.

    $self->{Catalog} = Pdf::Catalog->new;

    # Initialise colorspace. If there are no marks on the page then
    # this modules is not necessary and does not need to be loaded.

    if( eval{ $self->existColorSpace() } ) {
	$self->initColor();
    }
}

# }}}
# {{{ pdfOpen pdfClose

# Must be called before any writes to pdf file.  Writes a PDF file
# header to $path.  If $path is a reference then it must reference a
# file handle. Data is written to this file handle. Otherwise, if
# $path is not a reference, then a new file by the name $path is
# opened and all data is written to this file.

#no strict 'vars';
sub pdfOpen {
    my $self = shift;
    my $path = shift;

    if( !ref($path) ) {
        rename($path,"$path\~") if( -w $path && $self->{pdf_make_file_backup} );
#raman: commenting preparatory to nuking 
	#local($Pdf::File::sym) = $self->gensym;
	# open( __FH00, ">$path" ) 
	open($self->{FileHandle}, ">$path" )
	    || message_err( "Cannot open (output) pdf file '$path'" );
	# DRE: set binary mode for file handle
	# binmode __FH00;
	binmode $self->{FileHandle};
        #raman:Store  ref to a glob instead
	#$self->{FileHandle} = $Pdf::File::sym;
        #raman: replacing above with
        # $self->{FileHandle} =  \*__FH00;
        $self->{FileClose} = 1;
    } else {
	$self->{FileHandle} = $path;
    }

    # Write header information to pdf file

    $self->pdfPrint( "%PDF-$self->{pdf_language_version}\n" );
    $self->pdfPrint( "%\0342\0343\0317\0323\015\n" ); # This makes mailers think this is a binary file

    $self->{Print} = 1;
}


      #raman: This sub needs to be documented or nuked
#raman: commenting out preparatory to nuking       
# sub gensym {
#     my $pkg = @_ ? ref($_[0]) || $_[0] : "";
#     local *{$pkg . "::GLOB" . ++$Pdf::File::seq};
#     \delete ${$pkg . "::"}{'GLOB' . $Pdf::File::seq};
# }



# Writes remaining (unprinted) objects and PDF trailer to PDF file. If
# $self->{FileClose} is 1 then closes file.

sub pdfClose {
    my $self = shift;

    # Ensure that the header has been printed

    $self->{Print} || message_err( 'PDF file header has yet to be printed' );

    # Ensure we have an info object, and print it. If the user wants #
    # to set his own info fields then info() would have been called #
    # already. Otherwise info() sets the minimum number of fields.

    defined( $self->{Info} ) or $self->info();
    
    $self->{Info}->printObj( $self ); 
    

    # Ensure we have a catalog object, but do not print it until the very end!

    defined( $self->{Catalog} ) or
$self->{Catalog} = Pdf::Catalog->new;
    # Build and print Outline tree -- Not done yet
#see Outline.pm
    # Build and print Bead trees -- Not done yet

    # Build Pages tree. If there are no Page objects then you may be
    # able to get away with not loading Pages.pm and
    # Page.pm. buildPagesTree and printPagesTree are in Pages.pm

    if( $self->{PageIndex} > 0 ) {
	eval{ $self->existPages(); } || 
          message_err( 'Module Pages.pm not loaded' );
	$self->buildPagesTree();
	$self->printPagesTree;
    }

    # Print off ANY remaining objects.

    my $objref;
    foreach $objref ( @{$self->{ObjRef}} ) {
	$objref->printObj( $self );
    }

    # Print the catalog last
    
    $self->{Catalog}->printObj( $self ); 
    
    # Print trailer

    $self->_printTrailer;

    close $ {$self->{FileHandle}} if $self->{FileClose} == 1;
}

# }}}
# {{{ pdfClose helper routines

sub _printTrailer {
    my $self = shift;

    my($xrefOffset) = $self->{ByteCount};

    $self->_printXrefTable();

    # I could define a dictionary then print it. But what the heck.
    
    $self->pdfPrint( "trailer\n" );
    $self->pdfPrint( "<<\n" );
    $self->pdfPrint( sprintf( "/Size %d\n", $self->{ObjCount} + 1 ) );
    $self->pdfPrint( "/Root ", $self->{Catalog}->getObjNum, " 0 R\n" );
    $self->pdfPrint( "/Info ", $self->{Info}->getObjNum, " 0 R\n" );
    $self->pdfID();
    $self->pdfPrint( ">>\n" );
    $self->pdfPrint( "startxref\n" );
    $self->pdfPrint( "$xrefOffset\n" );
    $self->pdfPrint( "%%EOF\n" );
}

sub _printXrefTable {
    my $self = shift;
    
    $self->pdfPrint( "xref\n" );
    $self->pdfPrint( sprintf( "0 %d\n", $self->{ObjCount} + 1 ) );
    $self->pdfPrint( "0000000000 65535 f \n" );

    my $i;
    for( $i=1; $i<=$self->{ObjCount}; $i++ ) {
	$self->pdfPrint( sprintf( "%010d 00000 n \n", $self->{Xref}{$i} ) );
    }
}

#Converts a hex string that represents characters in $string

sub hexify {
    my ($string) = @_;
    my $hex;
    my $char;
    foreach $char (split(//, $string)) {
        $hex .= sprintf("%2x", ord($char));
    }
    return $hex;
}

#we make up the id from the current time and process id:

sub pdfID {
    my $self=shift;
    my $stamp =qx(date).$$;
    my $id = hexify ($stamp);
     $self->pdfPrint( "/ID [<$id><$id>]\n" );
}

# }}}
# {{{ setPageSize 

  #Usage:
  #$self->setPageSize (x,y)
#$self is a file object
  
sub setPageSize {
    my ($self) = shift;
    my ($x, $y) = @_;
    die "X dimension for page size not specified" unless defined ($x);
        die "Y dimension for page size not specified" unless defined ($y);
    $self->{pdf_xpsize} = $x;
    $self->{pdf_ypsize} = $y;
}

# }}}
# {{{ printing

# Use for all writes to pdf file. Prints @str to pdf file, and
# increments byte counter

# I need to test filehandle code to see if it works

sub pdfPrint {
    my $self = shift;
#print STDERR    ${$self->{FileHandle}},"\n";
      my $fh = ${$self->{FileHandle}};
    foreach (@_) {
	print $fh  $_;
	$self->{ByteCount} += length($_);
    }
}

# }}}
# {{{ beginObj endObj

# Start a new pdf object, and return the obj number.

sub beginObj {
    my $self = shift;
    ++$self->{ObjCount};
    $self->{Xref}{$self->{ObjCount}} = $self->{ByteCount};
    $self->pdfPrint( "$self->{ObjCount} 0 obj\n" );
    $self->{ObjCount};
}

sub endObj {
    my $self = shift;
    $self->pdfPrint( "endobj\n" );
}

# Reserve an object number, then print it out later using lateObj()
# Used for annotations and other situations where cyclical references
# are used.

sub reserveObj {
    my $self = shift;
    ++$self->{ObjCount};
}

sub lateObj {
    my $self = shift;
    my $obj = shift;
    $self->{Xref}{$obj} = $self->{ByteCount};
    $self->pdfPrint( "$obj 0 obj\n" );
    $obj;
}

# }}}
# {{{ Miscellaneous Pdf::File routines

# Adds an object ref (eg. ref to Annot, Font) to internal Obj list.

sub addObjRef {
    my $self = shift;
    my $objref = shift;
    push( @{$self->{ObjRef}}, $objref );
}

sub setPageRef {
    my $self = shift;
    my( $pagenum, $pageref ) = @_;
    eval{ $pageref->refType } eq 'PdfPage' ||
	message_err( 'Requires reference to existing PdfPage object' );
    $self->{Page}{$pagenum} = $pageref;
}

sub getPageRef {
    my $self = shift;
    $self->{Page}{$_[0]};
}

#raman: setColorSpaceRef is missing?

sub getColorSpaceRef {
    my $self = shift;
    $self->{ColorSpace}{$_[0]};
}

sub setFontRef {
    my $self = shift;
    my $name = shift;
    my $fontref = shift;
    eval{ $fontref->refType } eq 'PdfFont' ||
	message_err( "Reference is not a PdfFont" );
    $self->{Font}{$name} = $fontref;
}

sub getFontRef {
    my $self = shift;
    my $name = shift;
    my $fontref = $self->{Font}{$name};
}

# Modify global variables

sub setGlobal {
    my $self = shift;
    my( $key, $val ) = @_;

    defined( $val ) || message_err( 'Value undefined' );
    if( $key =~ /^pdf_/ && defined($self->{$key}) ) {
	$self->{$key} = $val;
    } else {
	message_err( "Global variable '$key' not defined or cannot be changed" );
    }
}

sub getGlobal {
    my $self = shift;
    my $key = shift;

    if( $key =~ /^pdf_/ && defined($self->{$key}) ) {
	return $self->{$key};
    } else {
	message_err( "Global variable '$key' not defined" );
    }
}

# }}}
# {{{ Emacs local variables  

### Local variables:
### major-mode: perl-mode
### perl-indent-level: 4
### voice-lock-mode: t
### folded-file: t
### End:

# }}}
1;
