# $Id: Info.pm,v 1.14 1996/05/31 16:10:04 raman Exp $
package Pdf::Info;
require 5.000;
use Pdf::Obj;
use Pdf::Std qw( message_err pdfDateString pdfEscape );
use strict qw ( subs vars refs );
@Pdf::Info::ISA = qw( Pdf::Obj );

# {{{  Info 

#############################################################################
# Info objects are defined
# If passed a reference then simply bless the ref into the current
# package. Otherwise create a new object and initialise it.

sub new {
    my $class = shift;
    my $self = shift;
    defined($self) || message_err( 'Requires a reference or producer name parameter' );
    if( !ref($self) ) {
	my $producer = $self;
	$self = Pdf::Obj->new( 'Title', 'Subject', 'Author',
				 'Creator', 'Producer',
				 'CreationDate', 'Keywords' );
	# Producer and CreationDate do not need pdfEscape, since
	# we know their values are okay
	$self->setObjDict( 'Producer',
		     Pdf::CosString->new( $producer ) ) if defined($producer);
	$self->setObjDict( 'CreationDate',
		     Pdf::CosString->new( pdfDateString(time,0) ) );
    }
    bless $self, $class;
}

BEGIN { # Static declaration for pdfInfo

    %Pdf::Info::field = ( 'title' => 'Title',
			 'author' => 'Author',
			 'creator' => 'Creator',
			 'subject' => 'Subject',
			 'keywords' => 'Keywords' );
}

sub refType {
    'PdfInfo';
}

sub setDict {
    my $self = shift;
    my(%p) = @_;

    my $text;
    my $key;
    foreach $key (keys %p) {
	$text = pdfEscape( $p{$key} );
	$key =~ tr/A-Z/a-z/;
	defined($Pdf::Info::field{$key}) ||
	    message_err( "Unknown info field '$key'" );
	$self->setObjDict( $Pdf::Info::field{$key},
		       Pdf::CosString->new( $text ) );
    }
}

# Nothing to validate

sub validate {
    my $self = shift;
}

# }}}
# {{{ package Pdf::File

package Pdf::File;

# User info routine.

sub info {
    my $self = shift;
    my(%p) = @_;

    my $info = Pdf::Info->new( $self->{pdf_info_producer} );
    $info->setObjNum( $self->reserveObj );
    $self->addObjRef( $info );
    $self->{Info} = $info;
    $info->setDict( %p );
    $info->validate;
}

# }}}
# {{{ Emacs local variables  

### Local variables:
### major-mode: perl-mode
### perl-indent-level: 4
### voice-lock-mode: t
### folded-file: t
### End:

# }}}
1;
