# $Id: Pages.pm,v 1.9 1996/05/31 16:14:16 raman Exp $
package Pdf::Pages;
require 5.000;
use Pdf::Std qw( message_err pdfEscape f0 f1 f2 f3 f4 assertFloat af0
                af1 af2 af3 af4 );
@ISA = qw( Pdf::Obj );
use strict qw ( subs vars refs );

# {{{ Pages core methods

#Constructor--

sub new {
    my $class = shift;
    my $self = Pdf::Obj->new( 'Type', 'Count', 'Parent', 'Kids',
			     'Resources', 'MediaBox', 'CropBox', 'Rotate',
			     'Dur', 'Hid', 'Trans' );
    $self->setObjDict( 'Type' => Pdf::CosName->new( 'Pages' ) );
    bless $self,$class;
}


sub refType {
    'PdfPages';
}



sub validate {
    my $self = shift;
    # May need to validate that there is a Parent, but not likely
    # since this is set automatically and not by the user
    1;
}

sub setDict {
    my $self = shift;
    my( %p ) = @_;

    my $key;
    my $valtype;
    foreach $key (keys %p) {
	$valtype = eval{ $p{$key}->refType };

	if( $key =~ /^media/i ) {
	    assertFloat( @{$p{$key}} ) == 4 || message_err( 'MediaBox must contain 4 numbers' );
	    $self->setObjDict( 'MediaBox', Pdf::CosArray->newInteger( af0( @{$p{$key}} ) ) );

	} elsif( $key =~ /^crop/i ) {
	    assertFloat( @{$p{$key}} ) == 4 || message_err( 'CropBox must contain 4 numbers' );
	    $self->setObjDict( 'CropBox', Pdf::CosArray->newInteger( af0( @{$p{$key}} ) ) );

	} elsif( $key =~ /^rot/i ) {
	    $p{$key}%90 == 0 || message_err( 'Rotate must be a multiple of 90' );
	    $self->setObjDict( 'Rotate', Pdf::CosInteger( f0( $p{$key} ) ) );

	} elsif( $key =~ /^dur/i ) {
            message_err( "Page '$key' cannot be a reference" )
              unless defined($valtype);
	    $self->setObjDict( 'Dur', Pdf::CosInteger( f3( $p{$key} )));

	} elsif( $key =~ /^hid/i ) {
            message_err( "Page '$key' cannot be a reference" )
              unless defined($valtype);
	    $self->setObjDict( 'Hid', Pdf::CosBoolean( $p{$key} ) );
	} elsif( $key =~ /^tra/i ) {
	    if( $valtype eq 'CosDict' ) {
		$self->setObjDict( 'Trans', $p{$key} );
	    } elsif( $valtype =~ /^Pdf/ ) {
		$self->setObjDict( 'Trans', Pdf::CosRef->new( $p{$key} ) );
	    } else {
		message_err( 'Page ', $key, ' requires a reference to a Cos Dict or Pdf Object' );
	    }

	} else {
	    message_err( "Parameter '$key' not supported by Pdf::Page::setDict()" );
	}
    }
}

# }}}
# {{{ Set objects and attributes

# Methods allow particular objects and attributes to be added to an
# initialised page.

sub setCount {
    my $self = shift;
    my $count = shift;
    !ref($count) || message_err( 'Count cannot be a reference' );
    $self->setObjDict( 'Count', Pdf::CosInteger->new( f0( $count ) ) );
}

sub addKid {
    my $self = shift;
    my $objref =shift;
    eval{ $objref->refType } eq 'CosArray' ||
	message_err( 'Reference to CosArray required' );
    $self->pushObjDictArray( 'Kids', $objref );
}

# }}}
# {{{ package Pdf::File

package Pdf::File;

# Call to check if Pages module has been loaded.

sub existPages {
    1;
}

sub buildPagesTree {
    my $self = shift;
    #Pravetz:
    # Create a new Pages object. Currently I am letting Page objects
    # define their own resources, and am letting Page objects inherit
    # their MediaBox from the Pages object. All this needs to be
    # polished up a bit. But I am concentrating on getting it working
    # first.
    

    my $pages_ref = Pdf::Pages->new;
    $pages_ref->setObjNum( $self->reserveObj );
    
    # Set the media box to default values
    # I need to rework this to find a new way for user to change these values.

    $pages_ref->setDict( 'MediaBox' => [ 0, 0,
				      $self->{pdf_xpsize},
				      $self->{pdf_ypsize} ] );

    # Add the pages object to the Catalog
    
    $self->{Catalog}->setDict( 'Pages' => $pages_ref );

    # Add all page refs to the Pages' Kids
    # For now use a flat pages tree. Later this will get more sophisticated.
    # Also for now build resource for each page individually.
    
    my $pagenum;
    my @pagerefs;
    for( $pagenum=1; defined($self->{Page}{$pagenum}); ++$pagenum ) {
	push( @pagerefs, Pdf::CosRef->new( $self->{Page}{$pagenum} ) );
	$self->{Page}{$pagenum}->buildResource;
    }
    if( scalar(@pagerefs) ) {
	$pages_ref->setCount( scalar(@pagerefs) );
	$pages_ref->setObjDict( 'Kids', Pdf::CosArray->new( @pagerefs ) );
    }
    1;				# must return 1 if successful
}

sub printPagesTree {
    my $self = shift;

    my $pagesref = $self->{Catalog}->getObjDict( 'Pages' )->getObjRef;
    my $cosarray = $pagesref->getObjDict( 'Kids' ) ;

    # Set Parent to Pages, and print Page objects

    if( defined( $cosarray ) ) {
	my($ref,$pageref);
	foreach $ref (@{$cosarray}) {
	    my $pageref = $ref->getObjRef(); # dereference
	    $pageref->setParent( $pagesref );
	    $pageref->printObj( $self );
	}
    }

    # Print Pages

    $pagesref->printObj( $self );
}

# }}}
# {{{ Emacs local variables  

### Local variables:
### major-mode: perl-mode
### perl-indent-level: 4
### voice-lock-mode: t
### folded-file: t
### End:

# }}}
1;
