# $Id: Std.pm,v 1.17 1996/05/31 16:18:43 raman Exp $
package Pdf::Std;
require 5.000;
require Exporter;
@ISA = qw(Exporter);
@EXPORT_OK = qw( message_err assertFloat assertFloatRange pdfEscape
                #string to float convertors 
		f0 f1 f2 f3 f4 af0 af1 af2 af3 af4
                #Output PDF Date String 
                pdfDateString );
use strict qw( subs vars refs );

# {{{  Begin

BEGIN {
    $Pdf::Std::trace_debug = 1;
}

# }}}
# {{{ Error Message Routine

# Prints $text and aborts with trace callback information. Will trace
# all the way out of the current package set (ie. any package that
# starts with 'Pdf::').

sub message_err {		# Private
    my($text) = @_;

    # close($pdf_ohandle) if( $pdf_file_close );
    print STDERR "ERROR: $text";
    my( @trace, @a );
    my $i;
    for( $i=0; ( (@a=caller($i))[0] =~ /^Pdf::/ && $i<10 ); ++$i ) {
	unshift( @trace, "line $a[2] ($a[1])\n\t$a[3]" ) if $Pdf::Std::trace_debug;
    }
    if( $a[0] !~ /^Pdf::/ && $i != 10) {
	print STDERR " ($a[0]::$a[1] line $a[2])\n";
    } else {
	print STDERR " (Callback information not available)\n";
    }
    print STDERR "Traceback:\n\tline $a[2] ($a[1])\n\t$a[3] ", join(' ', @trace), "\n"
	if $Pdf::Std::trace_debug && @trace;
    die "Output file may have been generated, but is unusable\n";
}

# }}}
# {{{  assertFloat assertFloatRange pdfEscape 

# If all arguments in list are valid floats then returns the number of
# floats in the list, otherwise returns 0.

sub assertFloat {
    my $total = 0;
    my $val;
    foreach $val ( @_ ) {
	# should assert that number is of form n.nn
        #raman: adding check for nn.n e +-nn
	++$total
          if( $val ne '' && $val =~ /^-?\d*.?\d*(e[+-]?\d+)?$/i );
    }
    ($total == scalar(@_))?$total:0;
}

# If all arguments in list are floats between $lo and $hi then returns
# the number of elements in the list, otherwise returns 0

sub assertFloatRange {
    my($lo,$hi,@v) = @_;

    ($lo,$hi) = ($hi,$lo) if $lo>$hi;

    my $total = 0;
    my $val;
    foreach $val ( @v ) {
	# should assert that number is of form n.nn
        #raman-- adding checks for scientific notation
	if( ( $val =~  /^-?\d*.?\d*(e[+-]\d+)?$/i ) && $val>=$lo && $val<=$hi ) {
	    ++$total;
	}
    }
    ($total == scalar(@v))?$total:0;
}

# Translates the text that users provide into strings to insert into
# pdf between parenthesis. Any special characters that the user wants
# to insert can be excaped using /032 syntax. These will be passed
# right through and interpreted by the pdf reader.

sub pdfEscape {
    my($text) = @_;
    #Escape backslash if not followed by a digit sequence 
    $text =~ s/\\(?!\\d+)/\\\\/g;
    # escape left and right parens 
    $text  =~ s/[()]/\\$&/g;
    # Write out non-printing chars using octal notation 
    $text =~ s/[\001-\033\200-\377]/sprintf("\\%03o",ord($&))/ge;
    $text;
}

# }}}
# {{{ Float to string converters

# Routines converts positive float numbers to a minimum length ascii
# string for insertion in pdf files. For most instances function f3
# gives enough precision, with 3 digits after the decimal
# place. Function f4 is used for specifying colors. These routines
# are optimised for speed.

sub f0 {
    sprintf( "%d", shift );
}

sub f1 {
    my $val = sprintf( "%.1f", shift );
    $val =~ s/0+$//;
    $val =~ s/\.$//;
    $val;
}

sub f2 {
    my $val = sprintf( "%.2f", shift );
    $val =~ s/0+$//;
    $val =~ s/\.$//;
    $val;
}

sub f3 {
    my $val = sprintf( "%.3f", shift );
    $val =~ s/0+$//;
    $val =~ s/\.$//;
    $val;
}

sub f4 {
    my $val = sprintf( "%.4f", shift );
    $val =~ s/0+$//;
    $val =~ s/\.$//;
    $val;
}

#do the same for arrays of floats

sub af0 {
    my(@rlist);
    foreach (@_) {
	push( @rlist, f0($_) );
    }
    @rlist;
}

sub af1 {
    my(@rlist);
    foreach (@_) {
	push( @rlist, f1($_) );
    }
    @rlist;
}

sub af2 {
    my(@rlist);
    foreach (@_) {
	push( @rlist, f2($_) );
    }
    @rlist;
}

sub af3 {
    my(@rlist);
    foreach (@_) {
	push( @rlist, f3($_) );
    }
    @rlist;
}

sub af4 {
    my(@rlist);
    foreach (@_) {
	push( @rlist, f4($_) );
    }
    @rlist;
}

# }}}
# {{{ Date and time formatting (uses PDF date format)

# Returns time in format defined by Adobe PDF Reference Manual.
# Format is D:YYYYMMDDHHmmSSOHH::mm'

sub pdfDateString {
  my($time) = @_ ;
  my(@lt,@gt);

  @lt = localtime($time);
  sprintf("D:%04d%02d%02d%02d%02d%02d%s%02d%02d", $lt[5]+1900, $lt[4]+1, $lt[3],
	  $lt[2], $lt[1], $lt[0], &pdfGzoneDiff()  );
}

# Returns ( plusminus, hours, minutes ) of local time zone relative to
# Greenwich time

sub pdfGzoneDiff {		# Private
    my($plusminus,$diff);
    my(@x) = localtime(0);
    if( $x[3] > 1 ) {
	$plusminus = "-";
	$diff = 60*(24-$x[2]) - $x[1];
    } else {
	$plusminus = "+";
	$diff = 60*$x[2] + $x[1];
    }
    ( $plusminus, int($diff/60), $diff%60 );
}

# }}}
# {{{ Emacs local variables  

### Local variables:
### major-mode: perl-mode
### perl-indent-level: 4
### voice-lock-mode: t
### folded-file: t
### End:

# }}}
1;
